require 'rspec'
require 'capybara'
require 'capybara/rspec'
require 'capybara/poltergeist'
require 'capybara/cucumber'

Capybara.register_driver :poltergeist do |app|
  Capybara::Poltergeist::Driver.new(app, :phantomjs_options => ['--ignore-ssl-errors=yes', '--ssl-protocol=any', '--load-images=no'])
end
Capybara.register_driver :poltergeist_debug do |app|
  Capybara::Poltergeist::Driver.new(app, :phantomjs_options => ['--ignore-ssl-errors=yes', '--ssl-protocol=any', '--load-images=yes'], :debug => true)
end

Capybara.default_driver = :poltergeist
Capybara.javascript_driver = :poltergeist
Capybara.run_server = false
Capybara.default_wait_time = 3

if ENV['DEBUG'] or ENV['VERBOSE']
  Capybara.default_driver = :poltergeist_debug
  Capybara.javascript_driver = :poltergeist_debug

end

if ENV['SERVER']
  SERVER=ENV['SERVER']
else
  SERVER='localhost'
end

# Screenshot any failed scenario
After do |scenario|
  if scenario.failed?
    if ENV['CUKE_SCREEN_DIR']
      screen_dir = ENV['CUKE_SCREEN_DIR']
    else
      screen_dir = './screenshots'
    end
    Dir::mkdir(screen_dir) if not File.directory?(screen_dir)
    screenshot = File.join(screen_dir, "FAILED_#{scenario.name.gsub(' ','_').gsub(/[^0-9A-Za-z_]/, '')}.png")
    screenshot_embed_filename = "./screenshots/FAILED_#{scenario.name.gsub(' ','_').gsub(/[^0-9A-Za-z_]/, '')}.png"
    page.driver.render(screenshot, :full => true)
    embed screenshot_embed_filename, 'image/png'
    if ENV['DEBUG'] or ENV['VERBOSE']
        page.driver.network_traffic.each { | request |
          puts "#{request.method} #{request.url} => \n" + request.response_parts.map { | response | "\t#{response.status} #{response.status_text}" }.join("\n")
        }
        puts page.html
    end
  end
end

Before do
  @params = {}
end

# Only login as monitor once, because it's slow
Before ('@asmonitor') do
  step %Q|I am logged in as "monitor" with password "monitor"|
end

# Only login as admin once, because it's slow
Before ('@asadmin') do
  step %Q|I am logged in to LogServer as "admin" with password "admin"|
end

# Head straight to nacoma - doing so might involve visiting configure
# FIXME: does it ever?
Before ('@nacoma') do
  @configuration = NagiosConfiguration.new("test/cucumber/templates/etc/.")
  step %Q|I enter nacoma|
end
After ('@nacoma') do |scenario|
  scenario.failed? ? @configuration.reset(false) : @configuration.reset
end

Before ('@configuration') do
  @configuration = NagiosConfiguration.new
end

After ('@configuration') do |scenario|
  #don't remove config dir if the test failed, useful for debugging
  scenario.failed? ? @configuration.reset(false) : @configuration.reset
end

Before ('@ninja_api_auth_enabled') do |scenario|
  $old_http_api_conf = File.read('/etc/op5/http_api.yml')
  File.open('/etc/op5/http_api.yml', 'w') { |file| file.write($old_http_api_conf.gsub('ninja: false', 'ninja: true')) }
end

After ('@ninja_api_auth_enabled') do |scenario|
  File.open('/etc/op5/http_api.yml', 'w') { |file| file.write($old_http_api_conf) }
end

